include "rose.bai"

' this function decomposes the object from
' a connected set of linesprites, to a set of moveto/drawto
' so that we are moving around indvidual lines.
'
' it will also precalculate the centrepoint atan2 angles, since we will absolutely 
' need this for performance....
'
' Params:
' obj - the LinesSprite format object
' world_scale - scale of the world
' point - where to explode
' x_impulse - base impulse to hit object with on the x axis
' y_impulse - base impulse to hit object with on the y axis
' x_random - additional random impulse for x
' y_random - additional random impulse for y
' break_apart - true/false - do we break the vectors?
function prepare_explosion2d(obj, world_scale, point, impulse, random, gravity, ground, break_apart)
  Struct ExpStruct {dest, accel, gravity, time, ground}
  if break_apart
    work_object = break_apart_object(obj, 2)
  else
    work_object = DeepCopy(obj)
  endif
  ' set up the acceleration table here for now
  dim work_accel[Ubound(work_object), 2]
  for vertex = 1 to Ubound(work_object)
    my_vector = {work_object[vertex, 2]-point[1], work_object[vertex, 3]-point[2]}
    my_vector = my_vector / norm(my_vector)
    my_vector = my_vector * world_scale
    work_accel[vertex, 1] = my_vector[1] * (impulse[1] + (rand() mod random[1]))
    work_accel[vertex, 2] = my_vector[2] * (impulse[2] + (rand() mod random[2]))
  next
  exp_obj = ExpStruct(work_object, work_accel, gravity*world_scale, 0.0, ground)
  return exp_obj
endfunction

function break_apart_object(obj, d)
  final_vcount = 0
  for j = 1 to Ubound(obj)
    if obj[j, 1] = DrawTo
      final_vcount = final_vcount + 2
    endif
  next
  dim workobj[final_vcount, d+1]
  v = 1
  last_v = {0,0,0}
  for j = 1 to Ubound(obj)
    if obj[j, 1] = DrawTo
      workobj[v, 1] = MoveTo
      workobj[v+1, 1] = DrawTo
      for x = 1 to d
        workobj[v, x+1] = lastv[x]
        workobj[v+1, x+1] = obj[j, x+1]
      next
      v = v + 2
    endif
    lastv = { obj[j, 2], obj[j, 3] }
  next
  return workobj
endfunction

' this function does the actual exploding
sub explode2d(workobj)
  ' update the time
  now = GetTickCount()
  if workobj.time = 0
    workobj.time = now
    return
  endif
  real_time = (now - workobj.time) / 960.0
  workobj.time = now

  ' accelerate the verticies
  for vertex = 1 to Ubound(workobj.dest)
    ' shift according to acccel*time
    workobj.dest[vertex, 2] = workobj.dest[vertex, 2] + workobj.accel[vertex, 1] * real_time
    workobj.dest[vertex, 3] = workobj.dest[vertex, 3] + workobj.accel[vertex, 2] * real_time
    ' we hit the ground - great!
    if workobj.dest[vertex, 3] < workobj.ground
      workobj.dest[vertex,3] = workobj.ground
      workobj.accel[vertex, 1] = 0
      workobj.accel[vertex, 2] = 0
    endif

    ' apply gravity 
    workobj.accel[vertex, 2] = workobj.accel[vertex, 2] - (workobj.gravity * real_time)
  next
endsub

call ScaleSprite(32)
call ReturnToOriginSprite()

' get the object
rose_object = rose()
' prepare the object for exploding
world_scale = 40
x_impulse = 2.5
y_impulse = 5.0
x_random = 2.5
y_random = 2.5
rose_explosion = prepare_explosion2d(rose_object, world_scale, {0,-400}, {x_impulse, y_impulse}, {x_random, y_random}, 9.8, -400, true)


' put it on the screen
call LinesSprite(rose_explosion.dest)
call SpriteClip(rose_explosion.dest, {{-450, -450}, {450, 450}})
controls = WaitForFrame(JoystickNone, Controller1, JoystickNone)
last_time = GetTickCount()
start_time = GetTicKCount()
while controls[1,3] = 0
  call explode2d(rose_explosion)
  ' wait for next frame :)
  controls = WaitForFrame(JoystickNone, Controller1, JoystickNone)
endwhile

